@@ -4,44 +4,42 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.9999_01';
+our $VERSION = '1.0001';
 
 use Carp              ();
 use File::Spec        ();
 use Exporter          ();
 use Data::MessagePack ();
-use Scalar::Util      ();
 
-use Text::Xslate::Util ();
-BEGIN {
-    # all the exportable functions are defined in ::Util
-    our @EXPORT_OK = qw(
-        mark_raw
-        unmark_raw
-        escaped_string
-        html_escape
-        uri_escape
-        html_builder
-    );
-    Text::Xslate::Util->import(@EXPORT_OK);
-}
+use Text::Xslate::Util qw(
+    mark_raw unmark_raw
+    html_escape escaped_string
+    uri_escape html_builder
+);
 
 our @ISA = qw(Text::Xslate::Engine);
 
-my $BYTECODE_VERSION = '1.6';
+our @EXPORT_OK = qw(
+    mark_raw
+    unmark_raw
+    escaped_string
+    html_escape
+    uri_escape
+    html_builder
+);
+
+my $BYTECODE_VERSION = '1.2';
 
 # $bytecode_version + $fullpath + $compiler_and_parser_options
 my $XSLATE_MAGIC   = qq{xslate;$BYTECODE_VERSION;%s;%s;};
 
 # load backend (XS or PP)
-my $use_xs = 0;
 if(!exists $INC{'Text/Xslate/PP.pm'}) {
     my $pp = ($Text::Xslate::Util::DEBUG =~ /\b pp \b/xms or $ENV{PERL_ONLY});
     if(!$pp) {
         eval {
             require XSLoader;
             XSLoader::load(__PACKAGE__, $VERSION);
-            $use_xs = 1;
         };
         die $@ if $@ && $Text::Xslate::Util::DEBUG =~ /\b xs \b/xms; # force XS
     }
@@ -49,7 +47,6 @@ if(!exists $INC{'Text/Xslate/PP.pm'}) {
         require 'Text/Xslate/PP.pm';
     }
 }
-sub USE_XS() { $use_xs }
 
 # workaround warnings about numeric when it is a developpers' version
 # it must be here because the bootstrap routine requires the under bar.
@@ -58,23 +55,19 @@ $VERSION =~ s/_//;
 # for error messages (see T::X::Util)
 sub input_layer { ref($_[0]) ? $_[0]->{input_layer} : ':utf8' }
 
-package Text::Xslate::Engine; # XS/PP common base class
+package Text::Xslate::Engine;
 
 use Text::Xslate::Util qw(
+    import_from
     make_error
-    dump
 );
 
-# constants
 BEGIN {
     our @ISA = qw(Exporter);
 
     my $dump_load = scalar($Text::Xslate::Util::DEBUG =~ /\b dump=load \b/xms);
     *_DUMP_LOAD = sub(){ $dump_load };
 
-    my $save_src = scalar($Text::Xslate::Util::DEBUG =~ /\b save_src \b/xms);
-    *_SAVE_SRC  = sub() { $save_src };
-
     *_ST_MTIME = sub() { 9 }; # see perldoc -f stat
 
     my $cache_dir = '.xslate_cache';
@@ -98,64 +91,42 @@ my %parser_option = (
 my %compiler_option = (
     syntax     => undef,
     type       => undef,
-    header     => undef, # template augment
-    footer     => undef, # template agument
-    macro      => undef, # template augment
+    header     => undef,
+    footer     => undef,
 );
 
 my %builtin = (
-    html_escape  => \&Text::Xslate::Util::html_escape,
+    raw          => \&Text::Xslate::Util::mark_raw,
+    html         => \&Text::Xslate::Util::html_escape,
     mark_raw     => \&Text::Xslate::Util::mark_raw,
     unmark_raw   => \&Text::Xslate::Util::unmark_raw,
-    uri_escape   => \&Text::Xslate::Util::uri_escape,
+    uri          => \&Text::Xslate::Util::uri_escape,
+    is_array_ref => \&_builtin_is_array_ref,
+    is_hash_ref  => \&_builtin_is_hash_ref,
 
-    is_array_ref => \&Text::Xslate::Util::is_array_ref,
-    is_hash_ref  => \&Text::Xslate::Util::is_hash_ref,
-
-    dump         => \&Text::Xslate::Util::dump,
-
-    # aliases
-    raw          => 'mark_raw',
-    html         => 'html_escape',
-    uri          => 'uri_escape',
+    dump       => \&Text::Xslate::Util::p,
 );
 
 sub default_functions { +{} } # overridable
 
-sub parser_option { # overridable
-    return \%parser_option;
-}
-
-sub compiler_option { # overridable
-    return \%compiler_option;
-}
-
-sub replace_option_value_for_magic_token { # overridable
-    #my($self, $name, $value) = @_;
-    #$value;
-    return $_[2];
-}
-
 sub options { # overridable
-    my($self) = @_;
     return {
         # name       => default
         suffix       => '.tx',
         path         => ['.'],
-        input_layer  => $self->input_layer,
+        input_layer  => ':utf8',
         cache        => 1, # 0: not cached, 1: checks mtime, 2: always cached
         cache_dir    => _DEFAULT_CACHE_DIR,
         module       => undef,
         function     => undef,
-        html_builder_module => undef,
         compiler     => 'Text::Xslate::Compiler',
 
         verbose      => 1,
         warn_handler => undef,
         die_handler  => undef,
 
-        %{ $self->parser_option },
-        %{ $self->compiler_option },
+        %parser_option,
+        %compiler_option,
     };
 }
 
@@ -166,7 +137,7 @@ sub new {
     my $options = $class->options;
     my $used    = 0;
     my $nargs   = scalar keys %args;
-    foreach my $key(keys %{$options}) {
+    while(my $key = each %{$options}) {
         if(exists $args{$key}) {
             $used++;
         }
@@ -176,9 +147,8 @@ sub new {
     }
 
     if($used != $nargs) {
-        my @unknowns = sort grep { !exists $options->{$_} } keys %args;
-        warnings::warnif(misc
-            => "$class: Unknown option(s): " . join ' ', @unknowns);
+        my @unknowns = grep { !exists $options->{$_} } keys %args;
+        warnings::warnif(misc => "$class: Unknown option(s): " . join ' ', @unknowns);
     }
 
     $args{path} = [
@@ -186,91 +156,49 @@ sub new {
             ref($args{path}) eq 'ARRAY' ? @{$args{path}} : $args{path}
     ];
 
-    my $arg_function= $args{function};
-
+    # function
     my %funcs;
+    $class->_merge_hash(\%funcs, $class->default_functions());
+
+    # 'module' overrides default functions
+    if(defined $args{module}) {
+        $class->_merge_hash(\%funcs, import_from(@{$args{module}}));
+    }
+
+    # 'function' overrides imported functons
+    $class->_merge_hash(\%funcs, $args{function});
+
+    # the following functions are not overridable
+    foreach my $name(keys %builtin) {
+        if(exists $funcs{$name}) {
+            warnings::warnif(redefine =>
+                "$class: You cannot redefine builtin function '$name',"
+                . " because it is embeded in the engine");
+        }
+        $funcs{$name} = $builtin{$name};
+    }
+
     $args{function} = \%funcs;
 
-    $args{template} = {}; # template structures
+    # internal data
+    $args{template} = {};
 
-    my $self = bless \%args, $class;
-
-    # definition of functions and methods
-
-    # builtin functions
-    %funcs = %builtin;
-    $self->_register_builtin_methods(\%funcs);
-
-    # per-class functions
-    $self->_merge_hash(\%funcs, $class->default_functions());
-
-    # user-defined functions
-    if(defined $args{module}) {
-        $self->_merge_hash(\%funcs,
-            Text::Xslate::Util::import_from(@{$args{module}}));
-    }
-
-    # user-defined html builder functions
-    if(defined $args{html_builder_module}) {
-        my $raw = Text::Xslate::Util::import_from(@{$args{html_builder_module}});
-        my $html_builders = +{
-            map {
-                ($_ => &Text::Xslate::Util::html_builder($raw->{$_}))
-            } keys %$raw
-        };
-        $self->_merge_hash(\%funcs, $html_builders);
-    }
-
-    $self->_merge_hash(\%funcs, $arg_function);
-
-    $self->_resolve_function_aliases(\%funcs);
-
-    return $self;
+    return bless \%args, $class;
 }
 
 sub _merge_hash {
     my($self, $base, $add) = @_;
-    foreach my $name(keys %{$add}) {
-        $base->{$name} = $add->{$name};
+    while(my($name, $body) = each %{$add}) {
+        $base->{$name} = $body;
     }
     return;
 }
 
-sub _resolve_function_aliases {
-    my($self, $funcs) = @_;
-
-    foreach my $f(values %{$funcs}) {
-        my %seen; # to avoid infinate loops
-        while(!( ref($f) or Scalar::Util::looks_like_number($f) )) {
-            my $v = $funcs->{$f} or $self->_error(
-               "Cannot resolve a function alias '$f',"
-               . " which refers nothing",
-            );
-
-            if( ref($v) or Scalar::Util::looks_like_number($v) ) {
-                $f = $v;
-                last;
-            }
-            else {
-                $seen{$v}++ and $self->_error(
-                    "Cannot resolve a function alias '$f',"
-                    . " which makes circular references",
-                );
-            }
-        }
-    }
-
-    return;
-}
-
-sub load_string { # called in render_string()
+sub load_string { # for <string>
     my($self, $string) = @_;
     if(not defined $string) {
         $self->_error("LoadError: Template string is not given");
     }
-    $self->note('  _load_string: %s', join '\n', split /\n/, $string)
-        if _DUMP_LOAD;
-    $self->{source}{'<string>'} = $string if _SAVE_SRC;
     $self->{string_buffer} = $string;
     my $asm = $self->compile($string);
     $self->_assemble($asm, '<string>', \$string, undef, undef);
@@ -290,39 +218,31 @@ sub find_file {
     my $orig_mtime;
     my $cache_mtime;
     foreach my $p(@{$self->{path}}) {
-        $self->note("  find_file: %s in  %s ...\n", $file, $p) if _DUMP_LOAD;
+        print STDOUT "  find_file: $p / $file ...\n" if _DUMP_LOAD;
 
-        my $cache_prefix;
+        my $path_id;
         if(ref $p eq 'HASH') { # virtual path
             defined(my $content = $p->{$file}) or next;
-            $fullpath = \$content;
-
-            # NOTE:
-            # Because contents of virtual paths include their digest,
-            # time-dependent cache verifier makes no sense.
-            $orig_mtime   = 0;
-            $cache_mtime  = 0;
-            $cache_prefix = 'HASH';
+            $fullpath   = \$content;
+            # TODO: we should provide a way to specify this mtime
+            #       (like as Template::Provider?)
+            $orig_mtime = $^T;
+            $path_id    = 'HASH';
         }
         else {
             $fullpath = File::Spec->catfile($p, $file);
             defined($orig_mtime = (stat($fullpath))[_ST_MTIME])
                 or next;
-            $cache_prefix = Text::Xslate::uri_escape($p);
-            if (length $cache_prefix > 127) {
-                # some filesystems refuse a path part with length > 127
-                $cache_prefix = $self->_digest($cache_prefix);
-            }
+            $path_id    = Text::Xslate::uri_escape($p);
         }
 
         # $file is found
         $cachepath = File::Spec->catfile(
             $self->{cache_dir},
-            $cache_prefix,
+            $path_id,
             $file . 'c',
         );
-        # stat() will be failed if the cache doesn't exist
-        $cache_mtime = (stat($cachepath))[_ST_MTIME];
+        $cache_mtime = (stat($cachepath))[_ST_MTIME]; # may fail, but doesn't matter
         last;
     }
 
@@ -330,11 +250,9 @@ sub find_file {
         $self->_error("LoadError: Cannot find '$file' (path: @{$self->{path}})");
     }
 
-    $self->note("  find_file: %s (mtime=%d)\n",
-        $fullpath, $cache_mtime || 0) if _DUMP_LOAD;
+    print STDOUT "  find_file: $fullpath (", ($cache_mtime || 0), ")\n" if _DUMP_LOAD;
 
     return {
-        name        => ref($fullpath) ? $file : $fullpath,
         fullpath    => $fullpath,
         cachepath   => $cachepath,
 
@@ -345,11 +263,11 @@ sub find_file {
 
 
 sub load_file {
-    my($self, $file, $mtime, $omit_augment) = @_;
+    my($self, $file, $mtime, $from_include) = @_;
 
-    local $self->{omit_augment} = $omit_augment;
+    local $self->{from_include} = $from_include;
 
-    $self->note("%s->load_file(%s)\n", $self, $file) if _DUMP_LOAD;
+    print STDOUT "load_file($file)\n" if _DUMP_LOAD;
 
     if($file eq '<string>') { # simply reload it
         return $self->load_string($self->{string_buffer});
@@ -371,10 +289,10 @@ sub load_file {
     return $asm;
 }
 
-sub slurp_template {
-    my($self, $input_layer, $fullpath) = @_;
+sub slurp {
+    my($self, $fullpath) = @_;
 
-    open my($source), '<' . $input_layer, $fullpath
+    open my($source), '<' . $self->{input_layer}, $fullpath
         or $self->_error("LoadError: Cannot open $fullpath for reading: $!");
     local $/;
     return scalar <$source>;
@@ -385,8 +303,6 @@ sub _load_source {
     my $fullpath  = $fi->{fullpath};
     my $cachepath = $fi->{cachepath};
 
-    $self->note("  _load_source: try %s ...\n", $fullpath) if _DUMP_LOAD;
-
     # This routine is called when the cache is no longer valid (or not created yet)
     # so it should be ensured that the cache, if exists, does not exist
     if(-e $cachepath) {
@@ -394,12 +310,10 @@ sub _load_source {
             or Carp::carp("Xslate: cannot unlink $cachepath (ignored): $!");
     }
 
-    my $source = $self->slurp_template($self->input_layer, $fullpath);
-    $self->{source}{$fi->{name}} = $source if _SAVE_SRC;
+    my $source = $self->slurp($fullpath);
 
     my $asm = $self->compile($source,
         file => $fullpath,
-        name => $fi->{name},
     );
 
     if($self->{cache} >= 1) {
@@ -408,35 +322,18 @@ sub _load_source {
         if(not -e $cachedir) {
             require File::Path;
             eval { File::Path::mkpath($cachedir) }
-                or Carp::croak("Xslate: Cannot prepare cache directory $cachepath (ignored): $@");
+                or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
         }
 
-        my $tmpfile = sprintf('%s.%d.d', $cachepath, $$, $self);
-
-        if (open my($out), ">:raw", $tmpfile) {
-            my $mtime = $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
+        if(open my($out), '>:raw', $cachepath) {
+            $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
                  Carp::carp("Xslate: Cannot close $cachepath (ignored): $!");
-                 unlink $tmpfile;
-            }
-            elsif (rename($tmpfile => $cachepath)) {
-                # set the newest mtime of all the related files to cache mtime
-                if (not ref $fullpath) {
-                    my $main_mtime = (stat $fullpath)[_ST_MTIME];
-                    if (defined($main_mtime) && $main_mtime > $mtime) {
-                        $mtime = $main_mtime;
-                    }
-                    utime $mtime, $mtime, $cachepath;
-                    $fi->{cache_mtime} = $mtime;
-                }
-                else {
-                    $fi->{cache_mtime} = (stat $cachepath)[_ST_MTIME];
-                }
+                 unlink $cachepath;
             }
             else {
-                Carp::carp("Xslate: Cannot rename cache file $cachepath (ignored): $!");
-                unlink $tmpfile;
+                $fi->{cache_mtime} = ( stat $cachepath )[_ST_MTIME];
             }
         }
         else {
@@ -444,8 +341,8 @@ sub _load_source {
         }
     }
     if(_DUMP_LOAD) {
-        $self->note("  _load_source: cache(mtime=%s)\n",
-            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
+        printf STDERR "  _load_source: cache(%s)\n",
+            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
     }
 
     return $asm;
@@ -466,8 +363,8 @@ sub _load_compiled {
     # see also tx_load_template() in xs/Text-Xslate.xs
     if(!( defined($fi->{cache_mtime}) and $self->{cache} >= 1
             and $threshold >= $fi->{orig_mtime} )) {
-        $self->note( "  _load_compiled: no fresh cache: %s, %s",
-            $threshold || 0, Text::Xslate::Util::p($fi) ) if _DUMP_LOAD;
+        printf "  _load_compiled: no fresh cache: %s, %s",
+            $threshold, Text::Xslate::Util::p($fi) if _DUMP_LOAD;
         $fi->{cache_mtime} = undef;
         return undef;
     }
@@ -495,11 +392,6 @@ sub _load_compiled {
     $unpacker->utf8($is_utf8);
 
     my @asm;
-    if($is_utf8) { # TODO: move to XS?
-        my $seed = "";
-        utf8::upgrade($seed);
-        push @asm, ['print_raw_s', $seed, __LINE__, __FILE__];
-    }
     while($offset < length($data)) {
         $offset = $unpacker->execute($data, $offset);
         my $c = $unpacker->data();
@@ -513,9 +405,9 @@ sub _load_compiled {
                 return undef; # purge the cache
             }
             if($dep_mtime > $threshold){
-                $self->note("  _load_compiled: %s(%s) is newer than %s(%s)\n",
+                printf "  _load_compiled: %s(%s) is newer than %s(%s)\n",
                     $c->[1],    scalar localtime($dep_mtime),
-                    $cachepath, scalar localtime($threshold) )
+                    $cachepath, scalar localtime($threshold)
                         if _DUMP_LOAD;
                 return undef; # purge the cache
             }
@@ -524,8 +416,8 @@ sub _load_compiled {
     }
 
     if(_DUMP_LOAD) {
-        $self->note("  _load_compiled: cache(mtime=%s)\n",
-            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
+        printf STDERR "  _load_compiled: cache(%s)\n",
+            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
     }
 
     return \@asm;
@@ -533,23 +425,13 @@ sub _load_compiled {
 
 sub _save_compiled {
     my($self, $out, $asm, $fullpath, $is_utf8) = @_;
-    my $mp = Data::MessagePack->new();
     local $\;
     print $out $self->_magic_token($fullpath);
-    print $out $mp->pack($is_utf8 ? 1 : 0);
-
-    my $newest_mtime = 0;
+    print $out Data::MessagePack->pack($is_utf8 ? 1 : 0);
     foreach my $c(@{$asm}) {
-        print $out $mp->pack($c);
-
-        if ($c->[0] eq 'depend') {
-            my $dep_mtime = (stat $c->[1])[_ST_MTIME];
-            if ($newest_mtime < $dep_mtime) {
-                $newest_mtime = $dep_mtime;
-            }
-        }
+        print $out Data::MessagePack->pack($c);
     }
-    return $newest_mtime;
+    return;
 }
 
 sub _magic_token {
@@ -557,28 +439,20 @@ sub _magic_token {
 
     $self->{serial_opt} ||= Data::MessagePack->pack([
         ref($self->{compiler}) || $self->{compiler},
-        $self->_filter_options_for_magic_token($self->_extract_options($self->parser_option)),
-        $self->_filter_options_for_magic_token($self->_extract_options($self->compiler_option)),
-        $self->input_layer,
-        [sort keys %{ $self->{function} }],
+        $self->_extract_options(\%parser_option),
+        $self->_extract_options(\%compiler_option),
+        [sort keys %{ $self->{function} } ],
     ]);
 
     if(ref $fullpath) { # ref to content string
-        $fullpath = join ':', ref($fullpath),
-            $self->_digest(${$fullpath});
+        require 'Digest/MD5.pm';
+        my $md5 = Digest::MD5->new();
+        $md5->add(${$fullpath});
+        $fullpath = join ':', ref($fullpath), $md5->hexdigest();
     }
     return sprintf $XSLATE_MAGIC, $fullpath, $self->{serial_opt};
 }
 
-sub _digest {
-    my($self, $content) = @_;
-    require 'Digest/MD5.pm'; # we don't want to create its namespace
-    my $md5 = Digest::MD5->new();
-    utf8::encode($content);
-    $md5->add($content);
-    return $md5->hexdigest();
-}
-
 sub _extract_options {
     my($self, $opt_ref) = @_;
     my @options;
@@ -590,35 +464,22 @@ sub _extract_options {
     return @options;
 }
 
-sub _filter_options_for_magic_token {
-    my($self, @options) = @_;
-    my @filterd_options;
-    while (@options) {
-        my $name  = shift @options;
-        my $value = $self->replace_option_value_for_magic_token($name, shift @options);
-        push(@filterd_options, $name => $value);
-    }
-    @filterd_options;
-}
-
-
-
 sub _compiler {
     my($self) = @_;
     my $compiler = $self->{compiler};
 
     if(!ref $compiler){
-        require Mouse;
-        Mouse::load_class($compiler);
+        require Any::Moose;
+        Any::Moose::load_class($compiler);
 
         my $input_layer = $self->input_layer;
         $compiler = $compiler->new(
             engine      => $self,
             input_layer => $input_layer,
-            $self->_extract_options($self->compiler_option),
+            $self->_extract_options(\%compiler_option),
             parser_option => {
                 input_layer => $input_layer,
-                $self->_extract_options($self->parser_option),
+                $self->_extract_options(\%parser_option),
             },
         );
 
@@ -632,17 +493,23 @@ sub _compiler {
 
 sub compile {
     my $self = shift;
-    return $self->_compiler->compile(@_,
-        omit_augment => $self->{omit_augment});
+    return $self->_compiler->compile(@_, from_include => $self->{from_include});
+}
+
+sub _builtin_is_array_ref {
+    return ref($_[0]) eq 'ARRAY';
+}
+
+sub _builtin_is_hash_ref {
+    return ref($_[0]) eq 'HASH';
 }
 
 sub _error {
     die make_error(@_);
 }
 
-sub note {
-    my($self, @args) = @_;
-    printf STDERR @args;
+sub dump :method {
+    goto &Text::Xslate::Util::p;
 }
 
 package Text::Xslate;
@@ -655,13 +522,18 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.9999_01.
+This document describes Text::Xslate version 1.0001.
 
 =head1 SYNOPSIS
 
     use Text::Xslate qw(mark_raw);
 
-    my $tx = Text::Xslate->new();
+    my $tx = Text::Xslate->new(
+        # the following options are optional.
+        path       => ['.'],
+        cache_dir  => "$ENV{HOME}/.xslate_cache",
+        cache      => 1,
+    );
 
     my %vars = (
         title => 'A list of books',
@@ -692,21 +564,14 @@ This document describes Text::Xslate version 1.9999_01.
 
 =head1 DESCRIPTION
 
-B<Xslate> is a template engine, tuned for persistent applications,
+B<Text::Xslate> is a template engine, tuned for persistent applications,
 safe as an HTML generator, and with rich features.
 
-There are a lot of template engines in CPAN, for example Template-Toolkit,
-Text::MicroTemplate, HTML::Template, and so on, but all of them have
-demerits at some points. This is why Xslate is developed and now it is
-well-honed as the standard template engine for web applications.
-
 The concept of Xslate is strongly influenced by Text::MicroTemplate
 and Template-Toolkit 2, but the central philosophy of Xslate is different
 from them. That is, the philosophy is B<sandboxing> that the template logic
 should not have no access outside the template beyond your permission.
 
-Other remarkable features are as follows:
-
 =head2 Features
 
 =head3 High performance
@@ -716,28 +581,44 @@ compiled into intermediate code, and then executed by the virtual machine,
 which is highly optimized for rendering templates. Thus, Xslate is
 much faster than any other template engines.
 
-The template roundup project by Sam Graham shows Text::Xslate got very
-high scores in I<instance_reuse> condition (i.e. for persistent application).
+Here is a result of F<benchmark/x-rich-env.pl> to compare various template
+engines in "rich" environment where applications are persistent and XS modules
+are available.
 
-=over
+    $ perl -Mblib benchmark/x-rich-env.pl
+    Perl/5.10.1 i686-linux
+    Text::Xslate/0.2002
+    Text::MicroTemplate/0.18
+    Text::MicroTemplate::Extended/0.11
+    Template/2.22
+    Text::ClearSilver/0.10.5.4
+    HTML::Template::Pro/0.9503
+    1..4
+    ok 1 - TT: Template-Toolkit
+    ok 2 - MT: Text::MicroTemplate
+    ok 3 - TCS: Text::ClearSilver
+    ok 4 - HTP: HTML::Template::Pro
+    Benchmarks with 'include' (datasize=100)
+              Rate     TT     MT    TCS    HTP Xslate
+    TT       129/s     --   -84%   -94%   -95%   -99%
+    MT       807/s   527%     --   -63%   -71%   -96%
+    TCS     2162/s  1580%   168%     --   -23%   -89%
+    HTP     2814/s  2087%   249%    30%     --   -85%
+    Xslate 19321/s 14912%  2295%   794%   587%     --
 
-=item The template roundup project
+According to this result, Xslate is 100+ times faster than Template-Toolkit.
+Text::MicroTemplate is a very fast template engine written in pure Perl, but
+XS-based modules, namely Text::ClearSilver, HTML::Template::Pro and Xslate
+are faster than Text::MicroTemplate. Moreover, Xslate is even faster than
+Text::ClearSilver and HTML::Template::Pro.
 
-L<http://illusori.co.uk/projects/Template-Roundup/>
+There are benchmark scripts in the F<benchmark/> directory.
 
-=item Perl Template Roundup October 2010 Performance vs Variant Report: instance_reuse
+=head3 Auto escaping to HTML meta characters
 
-L<http://illusori.co.uk/projects/Template-Roundup/201010/performance_vs_variant_by_feature_for_instance_reuse.html>
-
-=back
-
-There are some benchmarks in F<benchmark/> directory in the Xslate distribution.
-
-=head3 Smart escaping for HTML metacharacters
-
-All HTML metacharacters in template expressions which are interpolated into
-template texts by the engine are escaped automatically. This means that, by default,
-the output is not prone to XSS.
+All the HTML meta characters in template expressions the engine interpolates
+into template texts are automatically escaped, so the output has no
+possibility to XSS by default.
 
 =head3 Template cascading
 
@@ -747,9 +628,9 @@ but is more powerful.
 
 This mechanism is also called as template inheritance.
 
-=head3 Easiness to enhance
+=head3 Easy to enhance
 
-Xslate is ready to enhance. You can add functions and methods to the template
+Xslate is highly extensible. You can add functions and methods to the template
 engine and even add a new syntax via extending the parser.
 
 =head1 INTERFACE
@@ -780,46 +661,21 @@ checks the freshness of the original templates every time.
 If I<$level> E<gt>= 2, caches will be created but the freshness
 will not be checked.
 
-I<$level> == 0 uses no caches, which is provided for testing.
+I<$level> == 0 uses no caches. It's provided for testing.
 
 =item C<< cache_dir => $dir // "$ENV{HOME}/.xslate_cache" >>
 
 Specifies the directory used for caches. If C<$ENV{HOME}> doesn't exist,
 C<< File::Spec->tmpdir >> will be used.
 
-You B<should> specify this option for productions to avoid conflicts of
-template names.
+You B<should> specify this option on productions.
 
 =item C<< function => \%functions >>
 
 Specifies a function map which contains name-coderef pairs.
 A function C<f> may be called as C<f($arg)> or C<$arg | f> in templates.
 
-Note that those registered function have to return a B<text string>,
-not a binary string unless you want to handle bytes in whole templates.
-Make sure what you want to use returns whether text string or binary
-strings.
-
-For example, some methods of C<Time::Piece> might return a binary string
-which is encoded in UTF-8, so you'd like to decode their values.
-
-    # under LANG=ja_JP.UTF-8 on MacOSX (Darwin 11.2.0)
-    use Time::Piece;
-    use Encode qw(decode);
-
-    sub ctime {
-        my $ctime = Time::Piece->new->strftime; # UTF-8 encoded bytes
-        return decode "UTF-8", $ctime;
-    }
-
-    my $tx = Text::Xslate->new(
-        function => {
-            ctime => \&ctime,
-        },
-        ...,
-    );
-
-Built-in functions are described in L<Text::Xslate::Manual::Builtin>.
+There are builtin filters which are not overridable.
 
 =item C<< module => [$module => ?\@import_args, ...] >>
 
@@ -830,30 +686,26 @@ For example:
 
     # for function-based modules
     my $tx = Text::Xslate->new(
-        module => ['Digest::SHA1' => [qw(sha1_hex)]],
+        module => ['Time::Piece'],
     );
     print $tx->render_string(
-        '<: sha1_hex($x).substr(0, 6) :>',
-        { x => foo() },
-    ); # => 0beec7
+        '<: localtime($x).strftime() :>',
+        { x => time() },
+    ); # => Wed, 09 Jun 2010 10:22:06 JST
 
     # for bridge modules
     my $tx = Text::Xslate->new(
-        module => ['Text::Xslate::Bridge::Star'],
+        module => ['SomeModule::Bridge::Xslate'],
     );
     print $tx->render_string(
-        '<: $x.uc() :>',
-        { x => 'foo' },
-    ); # => 'FOO'
+        '<: $x.some_method() :>',
+        { x => time() },
+    );
 
 Because you can use function-based modules with the C<module> option, and
 also can invoke any object methods in templates, Xslate doesn't require
 specific namespaces for plugins.
 
-=item C<< html_builder_module => [$module => ?\@import_args, ...] >>
-
-Imports functions from I<$module>, wrapping each function with C<html_builder()>.
-
 =item C<< input_layer => $perliolayers // ':utf8' >>
 
 Specifies PerlIO layers to open template files.
@@ -874,9 +726,6 @@ If C<< $level >= 2 >>, all the possible errors will be warned.
 Specify the template suffix, which is used for C<cascade> and C<include>
 in Kolon.
 
-Note that this is used for static name resolution. That is, the compiler
-uses it but the runtime engine doesn't.
-
 =item C<< syntax => $name // 'Kolon' >>
 
 Specifies the template syntax you want to use.
@@ -889,10 +738,8 @@ This option is passed to the compiler directly.
 =item C<< type => $type // 'html' >>
 
 Specifies the output content type. If I<$type> is C<html> or C<xml>,
-smart escaping is applied to template expressions. That is,
-they are interpolated via the C<html_escape> filter.
-If I<$type> is C<text> smart escaping is not applied so that it is
-suitable for plain texts like e-mails.
+template expressions are interpolated via the C<html-escape> filter.
+If I<$type> is C<text>, template expressions are interpolated as they are.
 
 I<$type> may be B<html>, B<xml> (identical to C<html>), and B<text>.
 
@@ -900,7 +747,8 @@ This option is passed to the compiler directly.
 
 =item C<< line_start => $token // $parser_defined_str >>
 
-Specify the token to start line code as a string, which C<quotemeta> will be applied to. If you give C<undef>, the line code style is disabled.
+Specify the token to start line code as a string, which C<quotemeta> will be applied to. If you give C<undef> to this option, the line code style will be
+disabled.
 
 This option is passed to the parser via the compiler.
 
@@ -928,56 +776,53 @@ Specify the footer template files, which are inserted to the foot of each templa
 
 This option is passed to the compiler.
 
-=item C<< warn_handler => \&cb >>
-
-Specify the callback I<&cb> which is called on warnings.
-
-=item C<< die_handler => \&cb >>
-
-Specify the callback I<&cb> which is called on fatal errors.
-
 =back
 
 =head3 B<< $tx->render($file, \%vars) :Str >>
 
-Renders a template file with given variables, and returns the result.
+Renders a template file with variables, and returns the result.
 I<\%vars> is optional.
 
 Note that I<$file> may be cached according to the cache level.
 
 =head3 B<< $tx->render_string($string, \%vars) :Str >>
 
-Renders a template string with given variables, and returns the result.
+Renders a template string with variables, and returns the result.
 I<\%vars> is optional.
 
-Note that I<$string> is never cached, so this method should be avoided in
-production environment. If you want in-memory templates, consider the I<path>
-option for HASH references which are cached as you expect:
-
-    my %vpath = (
-        'hello.tx' => 'Hello, <: $lang :> world!',
-    );
-
-    my $tx = Text::Xslate->new( path => \%vpath );
-    print $tx->render('hello.tx', { lang => 'Xslate' });
-
-Note that I<$string> must be a text string, not a binary string.
+Note that I<$string> is never cached, so this method may not be suitable for
+productions.
 
 =head3 B<< $tx->load_file($file) :Void >>
 
-Loads I<$file> into memory for following C<render()>.
-Compiles and saves it as disk caches if needed.
+Loads I<$file> into memory for following C<render($file, \%vars)>.
+Compiles and saves it as caches if needed.
+
+It is a good idea to load templates before applications fork.
+Here is an example to to load all the templates which is in a given path:
+
+    my $path = ...;
+    my $tx = Text::Xslate->new(
+        path      => [$path],
+        cache_dir =>  $path,
+    );
+
+    find sub {
+        if(/\.tx$/) {
+            my $file = $File::Find::name;
+            $file =~ s/\Q$path\E .//xsm; # fix path names
+            $tx->load_file($file);
+        }
+    }, $path;
+
+    # fork and render ...
+
 
 =head3 B<< Text::Xslate->current_engine :XslateEngine >>
 
 Returns the current Xslate engine while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
-=head3 B<< Text::Xslate->current_vars :HashRef >>
-
-Returns the current variable table, namely the second argument of
-C<render()> while executing. Otherwise returns C<undef>.
-
 =head3 B<< Text::Xslate->current_file :Str >>
 
 Returns the current file name while executing. Otherwise returns C<undef>.
@@ -988,10 +833,6 @@ This method is significant when it is called by template functions and methods.
 Returns the current line number while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
-=head3 B<< Text::Xslate->print(...) :Void >>
-
-Adds the argument into the output buffer. This method is available on executing.
-
 =head2 Exportable functions
 
 =head3 C<< mark_raw($str :Str) :RawStr >>
@@ -1010,7 +851,7 @@ For example:
     # => Mailaddress: Foo &lt;foo@example.com&gt;
 
 This function is available in templates as the C<mark_raw> filter, although
-the use of it is strongly discouraged.
+the use of it is discouraged.
 
 =head3 C<< unmark_raw($str :Str) :Str >>
 
@@ -1038,14 +879,13 @@ This function is available in templates as the C<uri> filter.
 
 =head3 C<< html_builder { block } | \&function :CodeRef >>
 
-Wraps a block or I<&function> with C<mark_raw> so that the new subroutine
-will return a raw string.
+Wraps I<&function> with C<mark_raw> so that the new subroutine returns
+a raw string.
 
 This function is used to tell the xslate engine that I<&function> is an
 HTML builder that returns HTML sources. For example:
 
     sub some_html_builder {
-        my @args = @_;
         my $html;
         # build HTML ...
         return $html;
@@ -1065,7 +905,7 @@ The C<xslate(1)> command is provided as a CLI to the Text::Xslate module,
 which is used to process directory trees or to evaluate one liners.
 For example:
 
-    $ xslate -Dname=value -o dest_path src_path
+    $ xslate -D name=value -o dest_path src_path
 
     $ xslate -e 'Hello, <: $ARGV[0] :> wolrd!' Xslate
     $ xslate -s TTerse -e 'Hello, [% ARGV.0 %] world!' TTerse
@@ -1093,14 +933,6 @@ C<< %% ... >> line code, instead of C<< <: ... :> >> and C<< : ... >>.
 B<TTerse> is a syntax that is a subset of Template-Toolkit 2 (and partially TT3),
 which is explained in L<Text::Xslate::Syntax::TTerse>.
 
-=item HTMLTemplate
-
-There's HTML::Template compatible layers in CPAN.
-
-L<Text::Xslate::Syntax::HTMLTemplate> is a syntax for HTML::Template.
-
-L<HTML::Template::Parser> is a converter from HTML::Template to Text::Xslate.
-
 =back
 
 =head1 NOTES
@@ -1157,43 +989,16 @@ Augment modifiers.
 
 =item *
 
+Loop controls.
+
+=item *
+
 Default arguments and named arguments for macros.
 
-=item *
-
-External macros.
-
-Just idea: in the new macro concept, macros and external templates will be
-the same in internals:
-
-    : macro foo($lang) { "Hello, " ~ $lang ~ " world!" }
-    : include foo { lang => 'Xslate' }
-    : # => 'Hello, Xslate world!'
-
-    : extern bar 'my/bar.tx';     # 'extern bar $file' is ok
-    : bar( value => 42 );         # calls an external template
-    : include bar { value => 42 } # ditto
-
-=item *
-
-An "too-safe" HTML escaping filter which escape all the symbolic characters
-
 =back
 
 =cut
 
-=head1 RESOURCES
-
-WEB: L<http://xslate.org/>
-
-ML: L<http://groups.google.com/group/xslate>
-
-IRC: #xslate @ irc.perl.org
-
-PROJECT HOME: L<http://github.com/xslate/>
-
-REPOSITORY: L<http://github.com/xslate/p5-Text-Xslate/>
-
 =head1 BUGS
 
 All complex software has bugs lurking in it, and this module is no
@@ -1218,6 +1023,12 @@ Xslate command:
 
 L<xslate>
 
+The Xslate web site and public repository:
+
+L<http://xslate.org/>
+
+L<http://github.com/gfx/p5-Text-Xslate>
+
 Other template modules that Xslate has been influenced by:
 
 L<Text::MicroTemplate>
@@ -1242,10 +1053,6 @@ L<Template::Benchmark>
 
 L<http://xslate.org/benchmark.html>
 
-Papers:
-
-L<http://www.cs.usfca.edu/~parrt/papers/mvc.templates.pdf> -  Enforcing Strict Model-View Separation in Template Engines
-
 =head1 ACKNOWLEDGEMENT
 
 Thanks to lestrrat for the suggestion to the interface of C<render()>,
@@ -1256,10 +1063,10 @@ Thanks to tokuhirom for the ideas, feature requests, encouragement, and bug find
 
 Thanks to gardejo for the proposal to the name B<template cascading>.
 
-Thanks to makamaka for the contribution of Text::Xslate::PP.
-
 Thanks to jjn1056 to the concept of template overlay (now implemented as C<cascade with ...>).
 
+Thanks to makamaka for the contribution of Text::Xslate::PP.
+
 Thanks to typester for the various inspirations.
 
 Thanks to clouder for the patch of adding C<AND> and C<OR> to TTerse.
@@ -1276,18 +1083,6 @@ Thanks to Mons Anderson for the bug reports and patches.
 
 Thanks to hirose31 for the feature requests and bug reports.
 
-Thanks to c9s for the contribution of the documents.
-
-Thanks to shiba_yu36 for the bug reports.
-
-Thanks to kane46taka for the bug reports.
-
-Thanks to cho45 for the bug reports.
-
-Thanks to shmorimo for the bug reports.
-
-Thanks to ueda for the suggestions.
-
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
