@@ -3,17 +3,22 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '1.9999_01';
+our $VERSION = '1.0001';
+$VERSION =~ s/_//; # for developpers versions
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
 }
-
 use Text::Xslate::Util qw(
     $DEBUG
     p
 );
 
+use constant _PP_OPCODE  => scalar($DEBUG =~ /\b pp=opcode  \b/xms);
+use constant _PP_BOOSTER => scalar($DEBUG =~ /\b pp=booster \b/xms);
+use constant _PP_BACKEND =>   _PP_OPCODE  ? 'Opcode'
+                            : _PP_BOOSTER ? 'Booster'
+                            :               'Booster'; # default
 use constant _PP_ERROR_VERBOSE => scalar($DEBUG =~ /\b pp=verbose \b/xms);
 
 use constant _DUMP_LOAD => scalar($DEBUG =~ /\b dump=load \b/xms);
@@ -21,19 +26,13 @@ use constant _DUMP_LOAD => scalar($DEBUG =~ /\b dump=load \b/xms);
 use Text::Xslate::PP::Const qw(:all);
 use Text::Xslate::PP::State;
 use Text::Xslate::PP::Type::Raw;
-use Text::Xslate::PP::Opcode;
-use Text::Xslate::PP::Method;
+use Text::Xslate ();
 
-use Scalar::Util ();
-use overload     ();
-use Carp         ();
+use Carp ();
 
-# it must be loaded dynamically
-require Text::Xslate;
+require sprintf('Text/Xslate/PP/%s.pm', _PP_BACKEND);
 
-my $state_class = 'Text::Xslate::PP::Opcode';
-
-$VERSION =~ s/_//; # for developpers versions
+my $state_class = 'Text::Xslate::PP::' . _PP_BACKEND;
 
 if(_PP_ERROR_VERBOSE) {
     Carp->import('verbose');
@@ -63,13 +62,14 @@ our %html_escape = (
     '<' => '&lt;',
     '>' => '&gt;',
     '"' => '&quot;',
-    "'" => '&#39;', # IE8 doesn't support &apos; in title
+    "'" => '&apos;',
 );
 our $html_metachars = sprintf '[%s]', join '', map { quotemeta } keys %html_escape;
 
-sub _register_builtin_methods {
-    my($self, $funcs) = @_;
-    Text::Xslate::PP::Method::tx_register_builtin_methods($funcs);
+sub options {
+    my $options  = Text::Xslate::Engine->options;
+    $options->{ compiler } = 'Text::Xslate::' . (_PP_BACKEND eq 'Booster' ? 'PP::Compiler' : 'Compiler');
+    return $options;
 }
 
 #
@@ -103,7 +103,7 @@ sub render {
         Carp::croak( sprintf("Xslate: Template variables must be a HASH reference, not %s", $vars ) );
     }
 
-    my $st = $self->tx_load_template( $name, 0 );
+    my $st = tx_load_template( $self, $name, 0 );
 
     local $_orig_die_handler  = $SIG{__DIE__};
     local $_orig_warn_handler = $SIG{__WARN__};
@@ -117,10 +117,6 @@ sub current_engine {
     return defined($_current_st) ? $_current_st->engine : undef;
 }
 
-sub current_vars {
-    return defined($_current_st) ? $_current_st->vars : undef;
-}
-
 sub current_file {
     return defined($_current_st)
         ? $_current_st->code->[ $_current_st->{ pc } ]->{file}
@@ -133,19 +129,6 @@ sub current_line {
         : undef;
 }
 
-sub print {
-    shift;
-    if(defined $_current_st) {
-        foreach my $s(@_) {
-            $_current_st->print($s);
-        }
-    }
-    else {
-        Carp::croak('You cannot call print() method outside render()');
-    }
-    return '';
-}
-
 # >> copied and modified from Text::Xslate
 
 sub _assemble {
@@ -159,6 +142,16 @@ sub _assemble {
 
     my $st  = $state_class->new();
 
+    if(_PP_BACKEND eq 'Booster'){
+        if($asm->[0][0] eq '_ppbooster') {
+            my $ppbooster = shift @{$asm};
+            $st->{ booster_code } = Text::Xslate::PP::Booster->compile($ppbooster->[1]);
+        }
+        else {
+            Carp::croak("Oops: No booster code: ", p($asm));
+        }
+    }
+
     $st->symbol({ %{$self->{ function }} });
 
     my $tmpl = [];
@@ -182,7 +175,9 @@ sub _assemble {
 
     my $len = scalar( @$asm );
 
-    $st->push_frame('main', $len);
+    my $mainframe = tx_push_frame( $st );
+    $mainframe->[ Text::Xslate::PP::TXframe_NAME ]    = 'main';
+    $mainframe->[ Text::Xslate::PP::TXframe_RETADDR ] = $len;
 
     $st->code_len( $len );
 
@@ -221,7 +216,9 @@ sub _assemble {
             file   => $oi_file,
         };
 
-        $code->[ $i ]->{ exec_code } = $OPCODE[ $opnum ];
+        if(_PP_BACKEND eq 'Opcode') {
+            $code->[ $i ]->{ exec_code } = $OPCODE[ $opnum ];
+        }
 
         my $oparg = $OPARGS[ $opnum ];
 
@@ -344,46 +341,12 @@ sub _assemble {
         $s =~ s/($uri_unsafe_rfc3986)/sprintf '%%' . '%02X', ord $1/xmsgeo;
         return $s;
     }
-
-    sub is_array_ref { ref($_[0]) eq 'ARRAY' }
-    sub is_hash_ref  { ref($_[0]) eq 'HASH'  }
-    sub is_code_ref  { ref($_[0]) eq 'CODE'  }
-
-    sub merge_hash { +{ %{ $_[0] }, %{ $_[1] } } }
 }
 
 #
 # INTERNAL
 #
 
-sub tx_check_itr_ar {
-    my ( $st, $ar, $frame, $line ) = @_;
-    return $ar if ref($ar) eq 'ARRAY';
-
-    if ( defined $ar ) {
-        if(my $x = Text::Xslate::PP::sv_is_ref($ar, 'ARRAY', '@{}')) {
-            return $x;
-        }
-
-        $st->error( [$frame, $line],
-            "Iterator variables must be an ARRAY reference, not %s",
-            Text::Xslate::Util::neat( $ar ) );
-    }
-    return [];
-}
-
-sub sv_is_ref {
-    my($sv, $t, $ov) = @_;
-    return $sv if ref($sv) eq $t;
-
-    if(Scalar::Util::blessed($sv)
-        && (my $m = overload::Method($sv, $ov))) {
-        $sv = $sv->$m(undef, undef);
-        return $sv if ref($sv) eq $t;
-    }
-    return undef;
-}
-
 sub tx_sv_eq {
     my($x, $y) = @_;
     if ( defined $x ) {
@@ -451,6 +414,7 @@ sub tx_repeat {
     elsif(!Scalar::Util::looks_like_number($rhs)) {
         $_current_st->error(undef, "Repeat count must be a number, not %s",
             Text::Xslate::Util::neat($rhs));
+        return undef;
     }
     else {
         if( ref( $lhs ) eq TXt_RAW ) {
@@ -460,7 +424,16 @@ sub tx_repeat {
             return $lhs x $rhs;
         }
     }
-    return '';
+}
+
+sub tx_push_frame {
+    my ( $st ) = @_;
+
+    if ( $st->current_frame > 100 ) {
+        Carp::croak("Macro call is too deep (> 100)");
+    }
+
+    return $st->frame->[ $st->current_frame( $st->current_frame + 1 ) ] ||= [];
 }
 
 
@@ -547,20 +520,16 @@ sub tx_all_deps_are_fresh {
     return 1;
 }
 
-sub tx_execute {
+sub tx_execute { 
     my ( $st, $vars ) = @_;
     no warnings 'recursion';
 
     if ( $_depth > 100 ) {
         Carp::croak("Execution is too deep (> 100)");
     }
-    if(_PP_ERROR_VERBOSE and ref $st->{code}->[0]->{ exec_code } ne 'CODE') {
-        Carp::croak("Oops: Not a CODE reference: "
-            . Text::Xslate::Util::neat($st->{code}->[0]->{ exec_code }));
-    }
 
-    local $st->{pc}   = 0;
-    local $st->{vars} = $vars;
+    $st->{pc}   = 0;
+    $st->{vars} = $vars;
 
     local $_depth      = $_depth + 1;
     local $_current_st = $st;
@@ -568,12 +537,25 @@ sub tx_execute {
     local $st->{local_stack};
     local $st->{SP} = [];
 
-    local $st->{sa};
-    local $st->{sb};
-    local $st->{output} = '';
-    $st->{code}->[0]->{ exec_code }->( $st );
-    @{$st->{frame}->[-1]} = Text::Xslate::PP::TXframe_START_LVAR - 1;
-    return $st->{output};
+    if ( _PP_BACKEND eq 'Booster' ) {
+        if(_PP_ERROR_VERBOSE and ref $st->{ booster_code } ne 'CODE') {
+            Carp::croak("Oops: Not a CODE reference: "
+                . Text::Xslate::Util::neat($st->{ booster_code }));
+        }
+        return $st->{ booster_code }->( $st );
+    }
+    else {
+        if(_PP_ERROR_VERBOSE and ref $st->{code}->[0]->{ exec_code } ne 'CODE') {
+            Carp::croak("Oops: Not a CODE reference: "
+                . Text::Xslate::Util::neat($st->{code}->[0]->{ exec_code }));
+        }
+
+        local $st->{sa};
+        local $st->{sb};
+        $st->{output} = '';
+        $st->{code}->[0]->{ exec_code }->( $st );
+        return $st->{output};
+    }
 }
 
 
@@ -647,11 +629,11 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 1.9999_01.
+This document describes Text::Xslate::PP version 1.0001.
 
 =head1 DESCRIPTION
 
-This module implements a Text::Xslate runtime engine in pure Perl.
+This module implements Text::Xslate runtime engine in pure Perl.
 Normally it will be loaded if it fails to load XS. So you do not need
 to use this module explicitly.
 
@@ -666,14 +648,58 @@ If you want to use Text::Xslate::PP, however, you can use it.
 
 XS/PP mode might be switched with C<< $ENV{XSLATE} = 'pp' or 'xs' >>.
 
+From 0.1024 on, there are two pure Perl engines.
+C<Text::Xslate::PP::Booster>, enabled by C<< $ENV{XSLATE} = 'pp=booster' >>,
+generates optimized Perl code from intermediate code.
+C<Text::Xslate::PP::Opcode>, enabled by C<< $ENV{XSLATE} = 'pp=opcode' >>,
+executes intermediate code directly, emulating the virtual machine in pure Perl.
+
+PP::Booster is much faster than PP::Opcode, but it may be less stable.
+The default pure Perl engine is B<PP::Booster>, so if you run into problems,
+please try C<< $ENV{XSLATE} = 'pp=opcode' >>.
+
 C<< $ENV{XSLATE} = 'pp=verbose' } >> may be useful for debugging.
 
+=head1 NOTE
+
+=head2 Performance
+
+There might be cases where you cannot use XS modules.
+
+Here is a result of F<benchmark/x-poor-env.pl> to compare pure Perl template
+engines in poor environment where applications runs as CGI scripts and XS
+modules are not available.
+
+    $ perl -Mblib benchmark/x-poor-env.pl
+    Perl/5.10.1 i686-linux
+    Xslate backend: Booster
+    Text::Xslate/0.1058
+    Template/2.22
+    HTML::Template/2.9
+    Text::MicroTemplate/0.15
+    Text::MicroTemplate::Extended/0.11
+    1..3
+    ok 1 - TT: Template-Toolkit
+    ok 2 - MT: Text::MicroTemplate
+    ok 3 - HT: HTML::Template
+    Benchmarks with 'include' (datasize=100)
+             Rate     TT     HT Xslate     MT
+    TT     76.1/s     --   -60%   -61%   -82%
+    HT      189/s   149%     --    -3%   -56%
+    Xslate  196/s   158%     4%     --   -54%
+    MT      429/s   463%   126%   118%     --
+
+According to this result, PP::Booster is over 2 times faster than Template::Toolkit,
+and as fast as HTML::Template, but slower than Text::MicroTemplate.
+
 =head1 SEE ALSO
 
 L<Text::Xslate>
 
 L<Text::Xslate::PP::Opcode>
 
+L<Text::Xslate::PP::Booster>
+
 =head1 AUTHOR
 
 Text::Xslate::PP stuff is originally written by Makamaka Hannyaharamitu
